@@ -1,34 +1,29 @@
 import asyncio
-from functools import lru_cache
-import inspect
 import sys
 import traceback
 from typing import (
-    Any,
     Callable,
     Dict,
     Generator,
-    Hashable,
     Iterable,
     List,
-    NoReturn,
-    Optional,
-    Tuple,
+    Set,
     Type,
     Union,
 )
 
 from iterwrapper import IterWrapper as iw
 
-from .typing import T_Dispatcher, T_Dispatcher_Callable
+from graia.broadcast.entities.track_log import TrackLogType
+
+from .typing import T_Dispatcher
 
 from .entities.exectarget import ExecTarget
 
 from .interfaces.dispatcher import DispatcherInterface
-from .entities.decorater import Decorater
+from .entities.decorator import Decorator
 from .entities.dispatcher import BaseDispatcher
 from .entities.event import BaseEvent
-from .entities.inject_rule import BaseRule
 from .entities.listener import Listener
 from .entities.namespace import Namespace
 from .entities.signatures import Force, RemoveMe
@@ -42,19 +37,36 @@ from .exceptions import (
     RequirementCrashed,
     UnexistedNamespace,
 )
-from .interfaces.decorater import DecoraterInterface
+from .interfaces.decorator import DecoratorInterface
 from .utilles import (
     argument_signature,
     dispatcher_mixin_handler,
     group_dict,
-    isasyncgen,
-    isgenerator,
     printer,
     run_always_await_safely,
     cached_isinstance,
 )
 from .typing import T_Dispatcher
-from .zone import Zone
+
+
+def path_optimizer(logs, current_paths: Dict[str, List[List["T_Dispatcher"]]]):
+    current_path: List[List["T_Dispatcher"]] = None
+    current_path_set: Set["T_Dispatcher"] = None
+    has_failures: set = set()
+
+    for log in logs:
+        if log[0] is TrackLogType.LookupStart:
+            current_path = current_paths[log[1]]
+            current_path_set = current_paths[log[1] + "$set"]
+        elif log[0] is TrackLogType.LookupEnd:
+            current_path = None
+        elif log[0] is TrackLogType.Result and log[2] not in current_path_set:
+            current_path[0].append(log[2])
+            current_path_set.add(log[2])
+        elif log[0] is TrackLogType.Continue:
+            has_failures.add(log[1])
+
+    return has_failures
 
 
 class Broadcast:
@@ -64,45 +76,34 @@ class Broadcast:
     namespaces: List[Namespace]
     listeners: List[Listener]
 
-    # 规则驱动注入
-    dispatcher_inject_rules: List[BaseRule] = []
-
     dispatcher_interface: DispatcherInterface
 
     debug_flag: bool
 
-    use_dispatcher_statistics: bool
-    use_reference_optimization: bool
-
     def __init__(
         self,
         *,
         loop: asyncio.AbstractEventLoop = None,
         debug_flag: bool = False,
-        inject_rules: Optional[List[BaseRule]] = None,
-        use_dispatcher_statistics: bool = False,
-        use_reference_optimization: bool = False
     ):
         self.loop = loop or asyncio.get_event_loop()
         self.default_namespace = Namespace(name="default", default=True)
         self.debug_flag = debug_flag
         self.namespaces = []
         self.listeners = []
-        self.dispatcher_inject_rules = inject_rules or []
         self.dispatcher_interface = DispatcherInterface(self)
-
-        self.use_dispatcher_statistics = use_dispatcher_statistics
-
-        if not use_dispatcher_statistics and use_reference_optimization:
-            raise ValueError(
-                "the feature of reference optimization requires dispatcher statictics."
-            )
-        self.use_reference_optimization = use_reference_optimization
+        self.dispatcher_interface.execution_contexts[0].dispatchers.insert(
+            0, DecoratorInterface(self.dispatcher_interface)
+        )
 
         @self.dispatcher_interface.inject_global_raw
-        def _(interface: DispatcherInterface):
+        async def _(interface: DispatcherInterface):
             if interface.annotation is interface.event.__class__:
                 return interface.event
+            elif interface.annotation is Broadcast:
+                return interface.broadcast
+            elif interface.annotation is DispatcherInterface:
+                return interface
 
     def default_listener_generator(self, event_class) -> Iterable[Listener]:
         return (
@@ -113,14 +114,6 @@ class Broadcast:
             # .collect(list)  # collect to a whole list
         )
 
-    def addInjectionRule(self, rule: BaseRule) -> NoReturn:
-        if rule in self.dispatcher_inject_rules:
-            raise ValueError("this rule has already been added!")
-        self.dispatcher_inject_rules.append(rule)
-
-    def removeInjectionRule(self, rule: BaseRule) -> NoReturn:
-        self.dispatcher_inject_rules.remove(rule)
-
     async def layered_scheduler(
         self, listener_generator: Generator[Listener, None, None], event: BaseEvent
     ):
@@ -149,160 +142,87 @@ class Broadcast:
         post_exception_event: bool = False,
         print_exception: bool = True,
         enableInternalAccess: bool = False,
-        use_inline_generator: bool = False,
-        use_dispatcher_statistics: bool = False,
-        use_reference_optimization: bool = False,
     ):
         from .builtin.event import ExceptionThrowed
 
         is_exectarget = cached_isinstance(target, ExecTarget)
         is_listener = cached_isinstance(target, Listener)
 
-        use_dispatcher_statistics = (
-            self.use_dispatcher_statistics or use_dispatcher_statistics
-        )
-        use_reference_optimization = (
-            self.use_reference_optimization or use_reference_optimization
-        )
-
         if is_listener:
             if target.namespace.disabled:
                 raise DisabledNamespace(
                     "catched a disabled namespace: {0}".format(target.namespace.name)
                 )
 
-        if use_dispatcher_statistics and not is_exectarget:
-            use_dispatcher_statistics = False
-        if use_reference_optimization and not use_dispatcher_statistics:
-            raise ValueError(
-                "the feature of reference optimization requires dispatcher statictics."
-            )
-
-        # 先生成 dispatchers
-        _dispatchers = dispatcher_mixin_handler(event.Dispatcher)
-        # 开始暴力注入
-        if is_exectarget:
-            if target.inline_dispatchers:
-                _dispatchers = target.inline_dispatchers + _dispatchers
-            if is_listener:
-                if target.namespace.injected_dispatchers:
-                    _dispatchers = target.namespace.injected_dispatchers + _dispatchers
-        if dispatchers:
-            _dispatchers = dispatchers + _dispatchers
-
         target_callable = target.callable if is_exectarget else target
         parameter_compile_result = {}
+        complete_finished = False
 
+        track_logs = []
         async with self.dispatcher_interface.start_execution(
-            event, _dispatchers, use_inline_generator
+            event,
+            [
+                *(dispatchers or []),
+                *(target.namespace.injected_dispatchers if is_listener else []),
+                *(target.inline_dispatchers if is_exectarget else []),
+                *dispatcher_mixin_handler(event.Dispatcher),
+            ],
+            track_logs,
         ) as dii:
-            try:
-                catched_first_dispatcher = next(dii.dispatcher_pure_generator())
-            except StopIteration:
-                dei = DecoraterInterface(dii)  # pylint: disable=unused-variable
-                self.dispatcher_interface.execution_contexts[
-                    0
-                ].source.dispatchers.insert(0, dei)
-            else:
-                if not isinstance(catched_first_dispatcher, DecoraterInterface):
-                    dei = DecoraterInterface(dii)  # pylint: disable=unused-variable
-                    self.dispatcher_interface.execution_contexts[
-                        0
-                    ].source.dispatchers.insert(0, dei)
-            # Decorater 的 Dispatcher 已经注入, 没他事了
-
             if enableInternalAccess or (
                 is_exectarget and target.enable_internal_access
             ):
-                internal_access_mapping = {Broadcast: self, DispatcherInterface: dii}
-                if is_exectarget:
-                    internal_access_mapping[target.__class__] = target
-                if is_listener:
-                    internal_access_mapping[Namespace] = target.namespace
 
                 @dii.inject_execution_raw
-                def _(interface: DispatcherInterface):
-                    return internal_access_mapping.get(interface.annotation)
-
-            for injection_rule in self.dispatcher_inject_rules:
-                if injection_rule.check(event, dii):
-                    dii.execution_contexts[-1].dispatchers.insert(
-                        1, injection_rule.target_dispatcher
-                    )
-
-            if is_exectarget:
-                whole_statistics: Dict[
-                    str, Dict[T_Dispatcher, Tuple[int, int]]
-                ] = target.dispatcher_statistics["statistics"]
-            else:
-                whole_statistics = {}
+                async def _(interface: DispatcherInterface):
+                    if interface.annotation is target.__class__:
+                        return target
+                    elif interface.annotation is Namespace and is_listener:
+                        return target.namespace
 
             await dii.exec_lifecycle("beforeExecution")
-            await dii.exec_lifecycle("beforeDispatch")
             try:
-                for name, annotation, default in argument_signature(target_callable):
-                    statistics: Dict[
-                        T_Dispatcher, Tuple[int, int]
-                    ] = whole_statistics.setdefault(name, {})
-                    if use_dispatcher_statistics:
-                        if (
-                            use_reference_optimization
-                        ):  # 启用被动性质的优化特性 引用缓存(Reference Cache)
-                            for dispatcher, this_statistics in sorted(
-                                statistics.items(), key=lambda x: x[1][0] / x[1][1]
-                            ):
-                                this_statistics[1] += 1
-                                if dispatcher() is None:  # ref 在对象 dead 时返回 None.
-                                    del statistics[dispatcher]
-                                    continue
+                await dii.exec_lifecycle("beforeDispatch")
 
-                                try:
-                                    result = await dii.lookup_by_directly(
-                                        dispatcher(), name, annotation, default
-                                    )
-                                except RequirementCrashed:  # 忽略单个 Dispatcher 执行时若无法满足要求时所引发的错误
-                                    pass
-                                except:
-                                    traceback.print_exc()
-                                    raise
-                                else:
-                                    this_statistics[0] += 1
-                                    parameter_compile_result[name] = result
-                            else:
-                                if parameter_compile_result.get(name):
-                                    continue
-
-                        (
-                            parameter_compile_result[name],
-                            target_dispatcher,
-                            target_source,
-                            past_dispatchers,
-                        ) = await dii.lookup_param(
-                            name, annotation, default, enable_extra_return=True
-                        )
-
-                        # TODO: 判定 target_source 来源的可靠性(毕竟...)
-                        if target_source.source() is not None:
-                            # 1.成功给出结果的次数, 2.总共被 call 的次数
-                            statistics.setdefault(target_dispatcher, [0, 0])
-                            this_statistics = statistics[target_dispatcher]
-                            this_statistics[0] += 1
-                            this_statistics[1] += 1
-
-                            if len(statistics) > 1:
-                                past_set = set(statistics.keys())
-                                for past_dispatcher in past_dispatchers:
-                                    if past_dispatcher in past_set:
-                                        statistics[past_dispatcher][1] += 1
-                    else:
-                        parameter_compile_result[name] = await dii.lookup_param(
-                            name, annotation, default
-                        )
                 if is_exectarget:
-                    if target.headless_decoraters:
-                        for hl_d in target.headless_decoraters:
-                            await dii.lookup_param(None, None, hl_d)
-            except ExecutionStop:
+                    if target.maybe_failure:
+                        for name, annotation, default in argument_signature(
+                            target_callable
+                        ):
+                            if name not in target.param_paths:
+                                target.param_paths[name] = dii.init_dispatch_path()
+                                target.param_paths[name + "$set"] = set()
+                            parameter_compile_result[name] = await dii.lookup_param(
+                                name, annotation, default, target.param_paths[name]
+                            )
+                    else:
+                        for name, annotation, default in argument_signature(
+                            target_callable
+                        ):
+                            parameter_compile_result[
+                                name
+                            ] = await dii.lookup_param_without_log(
+                                name, annotation, default, target.param_paths[name]
+                            )
+
+                    for hl_d in target.headless_decorators:
+                        await dii.lookup_param_without_log(None, None, hl_d)
+                else:
+                    for name, annotation, default in argument_signature(
+                        target_callable
+                    ):
+                        parameter_compile_result[
+                            name
+                        ] = await dii.lookup_param_without_log(
+                            name, annotation, default, target.param_paths[name]
+                        )
+
+                complete_finished = True
+
+                result = await run_always_await_safely(
+                    target_callable, **parameter_compile_result
+                )
+            except (ExecutionStop, PropagationCancelled):
                 raise
             except RequirementCrashed:
                 traceback.print_exc()
@@ -314,46 +234,16 @@ class Broadcast:
                     self.postEvent(ExceptionThrowed(exception=e, event=event))
                 raise
             finally:
-                await dii.exec_lifecycle("afterDispatch")
-
-            await dii.exec_lifecycle("beforeTargetExec")
-
-            try:
-                result = await run_always_await_safely(
-                    target_callable, **parameter_compile_result
-                )
-            except ExecutionStop:
-                raise  # 直接抛出.
-            except PropagationCancelled:
-                raise  # 防止事件广播
-            except Exception as e:
-                if print_exception:
-                    traceback.print_exc()
-                if post_exception_event:  # 如果没有referrer, 则广播事件, 如果有则向上抛出(防止重复抛出事件)
-                    self.postEvent(ExceptionThrowed(exception=e, event=event))
-                raise
-            finally:
-                _, exception, tb = sys.exc_info()
-                await dii.exec_lifecycle("afterTargetExec", exception, tb)
-                await dii.exec_lifecycle("afterExecution", exception, tb)
-
-            gener = None
-            if [inspect.isgenerator, isgenerator][isinstance(result, Hashable)](result):
-                gener = result
-                try:
-                    result = next(gener)
-                except StopIteration as e:
-                    result = e.value
+                if is_exectarget and target.maybe_failure:
+                    has_failures = path_optimizer(track_logs, target.param_paths)
+                    target.maybe_failure.intersection_update(has_failures)
+                # print(track_logs)
+                if complete_finished:
+                    _, exception, tb = sys.exc_info()
+                    await dii.exec_lifecycle("afterTargetExec", exception, tb)
+                    await dii.exec_lifecycle("afterExecution", exception, tb)
                 else:
-                    dii.alive_generator_dispatcher[-1].append((gener, False))
-            elif [inspect.isasyncgen, isasyncgen][isinstance(result, Hashable)](result):
-                gener = result
-                try:
-                    result = await gener.__anext__()
-                except StopAsyncIteration:
-                    return  # value = None
-                else:
-                    dii.alive_generator_dispatcher[-1].append((gener, True))
+                    await dii.exec_lifecycle("afterDispatch")
 
             if result.__class__ is Force:
                 return result.content
@@ -362,6 +252,7 @@ class Broadcast:
                 if cached_isinstance(target, Listener):
                     if target in self.listeners:
                         self.listeners.pop(self.listeners.index(target))
+
             return result
 
     def postEvent(self, event: BaseEvent):
@@ -451,19 +342,13 @@ class Broadcast:
     def removeListener(self, target):
         self.listeners.remove(target)
 
-    def includeZone(self, target_zone: Zone):
-        if self.containNamespace(Zone.namespace.name):
-            raise ValueError("the zone cannot import because its name has been used.")
-        self.namespaces.append(target_zone.namespace)
-        self.listeners.extend(target_zone.listeners)
-
     def receiver(
         self,
         event: Union[str, Type[BaseEvent]],
         priority: int = 16,
         dispatchers: List[Type[BaseDispatcher]] = [],
         namespace: Namespace = None,
-        headless_decoraters: List[Decorater] = [],
+        headless_decorators: List[Decorator] = [],
         enable_internal_access: bool = False,
     ):
         if cached_isinstance(event, str):
@@ -483,7 +368,7 @@ class Broadcast:
                         inline_dispatchers=dispatchers,
                         priority=priority,
                         listening_events=[event],
-                        headless_decoraters=headless_decoraters,
+                        headless_decorators=headless_decorators,
                         enable_internal_access=enable_internal_access,
                     )
                 )
